﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2013 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System.Collections.Generic;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Analysis.SolutionCaching {
  public enum ProducedBy { Crossover, Mutation, CrossoverAndMutation, Move, Other };

  [Item("SolutionInformation", "Stores meta-information about a solution.")]
  [StorableClass]
  public class SolutionInformation<T> : Item {
    [Storable]
    public ProducedBy ProducedBy { get; set; }
    [Storable]
    public List<T> ParentList { get; set; }
    [Storable]
    public int Generation { get; set; }

    public SolutionInformation() { }

    [StorableConstructor]
    protected SolutionInformation(bool deserializing) : base(deserializing) { }
    protected SolutionInformation(SolutionInformation<T> original, Cloner cloner)
      : base(original, cloner) {
      this.ProducedBy = original.ProducedBy;
      this.ParentList = new List<T>(original.ParentList);
      this.Generation = original.Generation;
    }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new SolutionInformation<T>(this, cloner);
    }

    public override bool Equals(object obj) {
      SolutionInformation<T> pi = obj as SolutionInformation<T>;
      if (pi == null) return false;

      if (ProducedBy == pi.ProducedBy && Generation == pi.Generation) {
        if (pi.ParentList == null && ParentList == null) return true;
        if (pi.ParentList != null && ParentList == null ||
          pi.ParentList == null && ParentList != null) return false;
        if (pi.ParentList.Count != ParentList.Count) return false;

        foreach (var p in ParentList) {
          if (!pi.ParentList.Contains(p)) return false;
        }
        return true;
      } else {
        return false;
      }
    }
  }
}
